﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace VocaTest
{
    public partial class MainForm : Form
    {
        private Vocs vocs;
        private Boolean showExtraGridCols;

        // Drag-and-drop delegate type and member var
        private delegate void DelegateAppendFile(String[] fileNames);
        private DelegateAppendFile delegateAppendFile; 

        public MainForm()
        {
            InitializeComponent();
            this.Activated += new EventHandler(MainForm_Activated);
            this.FormClosing += new FormClosingEventHandler(MainForm_FormClosing);
            this.GotFocus += new EventHandler(MainForm_GotFocus);

            delegateAppendFile = new DelegateAppendFile(this.DragDropOpen);
            
            // Do we show the extra cols or not?
            ShowExtraGridCols = chkViewExtraGridCols.Checked;

            // Create vocs and set up the dataGridView
            CreateNewVocs();
            vocFileDSBindingSource.DataSource = vocs.DataSet;
            vocDataGridView.CellClick += new DataGridViewCellEventHandler(vocDataGridView_CellClick);
            vocDataGridView.Focus();
            vocDataGridView.Font = new Font(vocDataGridView.Font.Name, 10);
            vocDataGridView.DragDrop += new DragEventHandler(vocDataGridView_DragDrop);
            vocDataGridView.DragEnter += new DragEventHandler(vocDataGridView_DragEnter);
        }

        # region MainForm Event Handlers
        
        private void MainForm_Activated(object sender, EventArgs e)
        {
            // Always set focus to the gridView!
            vocDataGridView.Focus();
        }

        private void MainForm_GotFocus(object sender, EventArgs e)
        {
            // Always set focus to the gridView!
            vocDataGridView.Focus();
        }

        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            // User is closing --> Save files if changed
            if (vocs.IsDirty)
            {
                DialogResult result = ShowSaveFileQuestion();
                if (result == DialogResult.Cancel)
                {
                    e.Cancel = true;
                }
                else if (result == DialogResult.Yes)
                {
                    SaveVocsToFile(false);
                }
            }
        }

        #endregion //MainForm Event Handlers

        #region ToolStripMenu Event Handlers

        private void closeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            NewVocs();
        }

        private void newToolStripMenuItem_Click(object sender, EventArgs e)
        {
            NewVocs();
        }

        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (vocs.IsDirty)
            {
                DialogResult saveFileResult = ShowSaveFileQuestion();

                if (saveFileResult == DialogResult.Cancel)
                {
                    return;
                }
                else if (saveFileResult == DialogResult.Yes)
                {
                    SaveVocsToFile(false);
                }
            }

            DialogResult result = openVocFileDialog.ShowDialog(this);
            if (result == DialogResult.OK)
            {
                CreateNewVocs(openVocFileDialog.FileNames);
            }
        }

        private void saveAsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            SaveVocsToFile(true);
        }

        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            SaveVocsToFile(false);
        }

        private void startToolStripMenuItem_Click(object sender, EventArgs e)
        {
            StartTest();
        }

        private void printToolStripMenuItem_Click(object sender, EventArgs e)
        {
            PrintVocs();
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void startToolStripMenuItem_Click_1(object sender, EventArgs e)
        {
            StartTest();
        }

        private void toggleExtraColumnsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            UpdateExtraGridCols();
        }


        private void aboutVocaTestNETToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AboutDialog aboutDialog = new AboutDialog();
            aboutDialog.ShowDialog(this);
        }

        private void printPreviewToolStripMenuItem_Click(object sender, EventArgs e)
        {
            PrintPreview();
        }

        private void statisticsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ShowStats();
        }

        private void appendToolStripMenuItem_Click(object sender, EventArgs e)
        {
            DialogResult result = openVocFileDialog.ShowDialog(this);
            if (result == DialogResult.OK)
            {
                AppendVocs(openVocFileDialog.FileNames);
            }
        }

        #endregion // ToolStripMenu Event Handlers

        #region Button and Grid Event Handlers

        private void vocDataGridView_CellClick(object sender, DataGridViewCellEventArgs e)
        {
            if (e.ColumnIndex == -1)
            {
                // The user clicked the row header --> Select full row!
                vocDataGridView.SelectionMode = DataGridViewSelectionMode.FullRowSelect;
            }
            else
            {
                // The user clicked something else --> Select single cells!
                vocDataGridView.SelectionMode = DataGridViewSelectionMode.CellSelect;
            }
        }

        private void vocDataGridView_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.FileDrop))
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }

        private void vocDataGridView_DragDrop(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.FileDrop))
            {
                Array arr = (Array)e.Data.GetData(DataFormats.FileDrop);

                this.BeginInvoke(delegateAppendFile, arr);
                this.Activate();
            }
        }           

        private void startButton_Click(object sender, EventArgs e)
        {
            StartTest();
        }

        private void btnShowStats_Click(object sender, EventArgs e)
        {
            ShowStats();
        }

        private void btnPrint_Click(object sender, EventArgs e)
        {
            PrintVocs();
        }

        private void btnPrintPreview_Click(object sender, EventArgs e)
        {
            PrintPreview();
        }

        private void chkViewExtraGridCols_CheckedChanged(object sender, EventArgs e)
        {
            ShowExtraGridCols = chkViewExtraGridCols.Checked;
        }

        #endregion //ToolStripMenu Event Handlers

        #region Private Helper Functions

        private void ShowStats()
        {
            StatsDialog statsDiag = new StatsDialog(vocs);
            statsDiag.Show(this);
        }

        private DialogResult ShowSaveFileQuestion()
        {
            String fileName = vocs.FileName;
            if (String.IsNullOrWhiteSpace(fileName))
            {
                fileName = "New";
            }
            DialogResult result = MessageBox.Show(this, "Save file " + fileName, "Save File", MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question);
            return result;
        }

        private void CreateNewVocs()
        {
            vocs = new Vocs();
            vocs.Changed += new Vocs.ChangedEventHandler(vocs_Changed);
            vocDataGridView.DataSource = vocs.DataSet;
            vocDataGridView.DataMember = vocs.DataSet.Vocs.TableName;
            vocDataGridView.Refresh();
            UpdateFormText();
            vocDataGridView.Focus();
        }

        private void CreateNewVocs(String[] fileNames)
        {
            vocs = new Vocs(fileNames);
            vocs.Changed += new Vocs.ChangedEventHandler(vocs_Changed);
            vocDataGridView.DataSource = vocs.DataSet;
            vocDataGridView.DataMember = vocs.DataSet.Vocs.TableName;
            vocDataGridView.Refresh();
            UpdateFormText();
            vocDataGridView.Focus();
        }

        private void DragDropOpen(String[] fileNames)
        {
            if (fileNames.Length == 1 &&
                String.IsNullOrWhiteSpace(vocs.FileName) &&
                vocs.GetVocs(false, false).Count == 0)
            {
                // This is a "new" voclist without added words --> "Open" the drag-n-dropped file
                CreateNewVocs(fileNames);
            }
            else
            {
                // We're adding to an already open list --> "Append" the drag-n-dropped file(s)!
                AppendVocs(fileNames);
            }
        }

        private void AppendVocs(String[] fileNames)
        {
            vocs.AppendVocs(fileNames);
            UpdateFormText();
            vocDataGridView.Focus();
        }

        private void vocs_Changed(object sender, EventArgs e)
        {
            vocDataGridView.Refresh();
            UpdateFormText();
        }

        private void StartTest()
        {
            TestDialog.TestType testType = TestDialog.TestType.Normal;
            if (rdoMainLang1.Checked)
            {
                testType = TestDialog.TestType.Normal;
            }
            else if (rdoMainLang2.Checked)
            {
                testType = TestDialog.TestType.Backwards;
            }
            else if (rdoMainLangMixed.Checked)
            {
                testType = TestDialog.TestType.Mixed;
            }

            // Only test failed?
            Boolean onlyTestFailed = chkOnlyTestFailed.Checked;

            TestDialog tDiag = new TestDialog(vocs, testType, onlyTestFailed);
            vocDataGridView.Visible = false;
            tDiag.ShowDialog(this);
            vocDataGridView.Visible = true;

            // Focus the datagridview after closing the dialog!
            vocDataGridView.Focus();
        }

        private void SaveVocsToFile(Boolean showSaveFileDialog)
        {
            String fileName = vocs.FileName;

            if (showSaveFileDialog || String.IsNullOrWhiteSpace(fileName))
            {
                saveVocFileDialog.FileName = (String.IsNullOrWhiteSpace(fileName)) ? "New" : fileName;
                DialogResult result = saveVocFileDialog.ShowDialog(this);
                if (result != DialogResult.OK)
                {
                    // The user wants to abort --> Return here!
                    return;
                }

                fileName = saveVocFileDialog.FileName;
            }

            // We got here --> The user wants to save the file!
            vocs.WriteToFile(fileName);
        }

        private void UpdateFormText()
        {
            this.Text = vocs.FileName;
            if (String.IsNullOrWhiteSpace(this.Text))
            {
                this.Text = "New";
            }

            if (vocs.IsDirty)
            {
                this.Text += "*";
            }
            this.Text += " - " + System.Reflection.Assembly.GetExecutingAssembly().GetName().Name;
        }

        private void PrintVocs()
        {
            DialogResult result = printDialog1.ShowDialog(this);
            if (result == DialogResult.OK)
            {
                VocsPrintDocument txtPrintDoc = new VocsPrintDocument(vocs);
                txtPrintDoc.Print();
            }

            vocDataGridView.Focus();
        }

        private void PrintPreview()
        {
            PrintPreviewDialog printPreview = new PrintPreviewDialog();
            printPreview.Document = new VocsPrintDocument(vocs);
            printPreview.Icon = System.Drawing.Icon.ExtractAssociatedIcon(Application.ExecutablePath);
            printPreview.WindowState = FormWindowState.Maximized;
            printPreview.ShowDialog(this);

            vocDataGridView.Focus();
        }

        private void UpdateExtraGridCols()
        {
            ShowExtraGridCols = toggleExtraColumnsToolStripMenuItem.Checked;
        }

        private void NewVocs()
        {
            if (vocs.IsDirty)
            {
                DialogResult result = ShowSaveFileQuestion();
                if (result == DialogResult.Cancel)
                {
                    // Do nothing!
                    return;
                }
                else if (result == DialogResult.Yes)
                {
                    SaveVocsToFile(false);
                }
            }

            // We got here --> Create a new VocList!
            CreateNewVocs();
        }

        #endregion // Private Helper Functions

        #region Private Properties

        private Boolean ShowExtraGridCols
        {
            get
            {
                return showExtraGridCols;
            }
            set
            {
                toggleExtraColumnsToolStripMenuItem.Checked = value;
                chkViewExtraGridCols.Checked = value;

                vocDataGridView.Columns["LastResultColumn"].Visible = value;
                vocDataGridView.Columns["TimesPassedColumn"].Visible = value;
                vocDataGridView.Columns["TimesRunColumn"].Visible = value;

                showExtraGridCols = value;
            }
        }

        #endregion // Private Properties        
    }
}
