﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace VocaTest
{
    /// <summary>
    /// Displays a dialog used to perform a test
    /// </summary>
    public partial class TestDialog : Form
    {
        public enum TestType
        {
            Normal,
            Backwards,
            Mixed
        }

        private State state;
        private List<Vocs.Voc> vocList;
        private IEnumerator<Vocs.Voc> vocListEnumerator;

        private Int32 currentVocNum;
        private Int32 numPassed;
        private Int32 numFailed;
        private String currentVocQuestion;
        private String currentVocAnswer;

        private enum State
        {
            TestRunning,
            Close
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="TestDialog"/> class.
        /// </summary>
        /// <param name="vocs">The vocs.</param>
        /// <param name="isBackwardsTest">if set to <c>true</c> the test should be performed "backwards", i.e. from Lang1--&gt;Lang2 instead of from Lang2--&gt;Lang1.</param>
        /// <param name="onlyTestFailed">if set to <c>true</c>, only test vocs that failed on the last test.</param>
        public TestDialog(Vocs vocs, TestType testType, Boolean onlyTestFailed)
        {
            InitializeComponent();
            this.FormClosing += new FormClosingEventHandler(TestDialog_FormClosing);
            this.Shown += new EventHandler(TestDialog_Shown);

            if (vocs == null)
            {
                throw new ArgumentException();
            }

            Vocs = vocs;
            TypeOfTest = testType;
            OnlyTestFailed = onlyTestFailed;

            vocList = Vocs.GetVocs(onlyTestFailed, true);
            vocListEnumerator = vocList.GetEnumerator();
            currentVocNum = 0;
            numPassed = 0;
            numFailed = 0;
            state = State.TestRunning;
        }

        private void TestDialog_Shown(object sender, EventArgs e)
        {
            if (Vocs == null)
            {
                this.Close();
                return;
            }

            NextVoc();
            UpdateGuiComponents();
        }

        private Vocs Vocs
        {
            get;
            set;
        }

        private Vocs.Voc CurrentVoc
        {
            get
            {
                return vocListEnumerator.Current;
            }
        }

        private TestType TypeOfTest
        {
            get;
            set;
        }

        private Boolean OnlyTestFailed
        {
            get;
            set;
        }

        private void NextVoc()
        {
            Boolean ok = vocListEnumerator.MoveNext();
            if (ok)
            {
                currentVocNum++;

                switch (TypeOfTest)
                {
                    case TestType.Normal:
                        {
                            currentVocQuestion = CurrentVoc.Language1;
                            currentVocAnswer = CurrentVoc.Language2;
                            break;
                        }
                    case TestType.Backwards:
                        {
                            currentVocQuestion = CurrentVoc.Language2;
                            currentVocAnswer = CurrentVoc.Language1;
                            break;
                        }
                    case TestType.Mixed:
                        {
                            Random rand = new Random();
                            if (rand.Next(2) == 0)
                            {
                                currentVocQuestion = CurrentVoc.Language1;
                                currentVocAnswer = CurrentVoc.Language2;
                            }
                            else
                            {
                                currentVocQuestion = CurrentVoc.Language2;
                                currentVocAnswer = CurrentVoc.Language1;
                            }

                            break;
                        }
                }
            }
            UpdateState();
        }

        private void UpdateState()
        {
            switch (state)
            {
                case State.TestRunning:
                    {
                        if (CurrentVoc == null)
                        {
                            state = State.Close;
                        }
                        break;
                    }
                case State.Close:
                    {
                        this.DialogResult = DialogResult.OK;
                        this.Close();
                        break;
                    }
            }

            UpdateGuiComponents();
        }

        private void UpdateGuiComponents()
        {
            switch (state)
            {
                case State.TestRunning:
                    {
                        lblCurrentVoc.Text = currentVocNum + "/" + vocList.Count;
                        lblNumPassed.Text = numPassed.ToString();
                        lblNumFailed.Text = numFailed.ToString();
                        txtTranslation.Text = null;
                        txtTranslation.Focus();
                        txtWordToTranslate.Text = currentVocQuestion;
                        break;
                    }
                case State.Close:
                    {
                        lblCurrentVoc.Text = currentVocNum + "/" + vocList.Count;
                        lblNumPassed.Text = numPassed.ToString();
                        lblNumFailed.Text = numFailed.ToString();
                        txtTranslation.Text = null;
                        txtTranslation.ReadOnly = true;
                        txtWordToTranslate.Text = null;
                        btnNext.Text = "Done";
                        btnNext.Focus();
                        break;
                    }
            }
        }

        private void btnNext_Click(object sender, EventArgs e)
        {
            if (CurrentVoc != null)
            {
                Boolean result = txtTranslation.Text.Equals(currentVocAnswer, StringComparison.InvariantCultureIgnoreCase);
                if (result)
                {
                    numPassed++;
                }
                else
                {
                    numFailed++;
                    WrongAnswerPopUp popup = new WrongAnswerPopUp(txtWordToTranslate.Text, txtTranslation.Text, currentVocAnswer);
                    popup.ShowDialog(this);
                }

                Vocs.NotifyResult(CurrentVoc.ID, result);
            }

            NextVoc();
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.Cancel;
            this.Close();
        }

        private void TestDialog_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (vocList.Count > 0 &&
                this.DialogResult == DialogResult.OK)
            {
                Vocs.SaveStats(numFailed + numPassed, numPassed);
            }
        }
    }
}
