﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data;
using System.IO;

namespace VocaTest
{
    /// <summary>
    /// Wrapper class used to create an abstraction for the representation och Vocs and statistics for the vocs
    /// </summary>
    public class Vocs
    {
        /// <summary>
        /// Helper class used to represents a single voc.
        /// Used to abstract away the details of the actual voc implementation
        /// </summary>
        public class Voc
        {
            private VocFileDS.VocsRow vocRow;

            public Voc(VocFileDS.VocsRow vocRow)
            {
                this.vocRow = vocRow;
            }

            public Int32 ID
            {
                get
                {
                    return vocRow.ID;
                }
            }

            public String Language1
            {
                get
                {
                    return vocRow.Language1;
                }
            }

            public String Language2
            {
                get
                {
                    return vocRow.Language2;
                }
            }
        }

        private VocFileDS ds;

        #region Public Functions

        /// <summary>
        /// Delegate for listeners to the Changed event
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        public delegate void ChangedEventHandler(object sender, EventArgs e);

        /// <summary>
        /// Occurs when the Vocs (or the statistics for the vocs) has changed in memory.
        /// </summary>
        public event ChangedEventHandler Changed;

        /// <summary>
        /// Initializes a new instance of the <see cref="Vocs"/> class.
        /// </summary>
        public Vocs()
        {
            ds = new VocFileDS();
            ds.Vocs.RowChanged += new DataRowChangeEventHandler(DsRow_Changed);
            ds.Statistics.RowChanged += new DataRowChangeEventHandler(DsRow_Changed);
            ds.AcceptChanges();
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="Vocs"/> class.
        /// </summary>
        /// <param name="fileNames">Names of the files.</param>
        public Vocs(String[] fileNames)
            : this()
        {
            try
            {
                if (fileNames.Length == 1)
                {
                    // We're opening a single file --> Just open it!
                    ds.ReadXml(fileNames[0], XmlReadMode.ReadSchema);
                    ds.AcceptChanges();
                    FileName = fileNames[0];
                }
                else
                {
                    // We're opening multiple files --> Just append them all to whatever voclist we've got!
                    AppendVocs(fileNames);
                }
            }
            catch (Exception)
            {
                // Probably failed when opening the file...nothing to do here in either case
            }
        }

        /// <summary>
        /// Appends new vocs to the existing ds
        /// </summary>
        /// <param name="fileNames">The file names to append vocs from</param>
        public void AppendVocs(String[] fileNames)
        {
            foreach (String fileName in fileNames)
            {
                // Load each file into a ds
                VocFileDS tempds = new VocFileDS();

                try
                {
                    tempds.ReadXml(fileName, XmlReadMode.ReadSchema);
                }
                catch (Exception)
                {
                    // Failed to read the xml file --> Try the next one!
                    continue;
                }

                // Copy all rows from the temp ds to the "real" ds.
                // This is a bit ugly but merge won't work because the primary key is not unique between files!
                foreach (VocFileDS.VocsRow row in tempds.Vocs)
                {
                    ds.Vocs.AddVocsRow(row.Language1,
                                       row.Language2,
                                       row.LastResult,
                                       row.TimesRun,
                                       row.TimesPassed);
                }
            }
        }

        /// <summary>
        /// Writes to file.
        /// </summary>
        public void WriteToFile()
        {
            ds.WriteXml(FileName);
            ds.AcceptChanges();
            OnChanged(EventArgs.Empty);
        }

        /// <summary>
        /// Writes to file.
        /// </summary>
        /// <param name="fileName">Name of the file.</param>
        public void WriteToFile(String fileName)
        {
            FileName = fileName;
            WriteToFile();
        }

        /// <summary>
        /// Saves stat info
        /// </summary>
        /// <param name="saveStats">if set to <c>true</c>, statistics are saved.</param>
        public void SaveStats(Int32 numTotalVocs, Int32 numPassed)
        {
            if (numTotalVocs > 0)
            {
                VocFileDS.StatisticsRow statRow = ds.Statistics.NewStatisticsRow();
                statRow.NumTotalVocs = numTotalVocs;
                statRow.NumPassedVocs = numPassed;
                statRow.RunDateTime = DateTime.Now;
                ds.Statistics.Rows.Add(statRow);
            }
        }

        /// <summary>
        /// Gets the vocs.
        /// </summary>
        /// <param name="onlyFailed">if set to <c>true</c> [only failed].</param>
        /// <param name="randomOrder">if set to <c>true</c> [random order].</param>
        /// <returns></returns>
        public List<Vocs.Voc> GetVocs(Boolean onlyFailed, Boolean randomOrder)
        {
            List<Vocs.Voc> vocList = new List<Voc>();

            VocFileDS.VocsRow[] vocArr = (VocFileDS.VocsRow[])((onlyFailed) ? ds.Vocs.Select("LastResult = 0") : ds.Vocs.ToArray());

            if (randomOrder)
            {
                Random randNr = new Random();
                Int32[] keyArr = new Int32[vocArr.Length];
                for (int i = 0; i < keyArr.Length; i++)
                {
                    keyArr[i] = randNr.Next();
                }

                Array.Sort(keyArr, vocArr);

                // The cast below is needed due to: (see http://igoro.com/archive/puzzling-over-arrays-and-enumerators-in-c/)
                // - Arrays are old and were implemented i .NET before there were support for generics
                // - Therefore, the GetEnumerator return an SZArrayEnumerator which is dows NOT implement IEnumerator (!)
                // - When generics were added, another GetEnumerator() was added that returns an IEnumerator as an explicit interface implementation
                // - In order to use the "new" GetEnumerator(), we have to explicitly vast the array to an enumerator before the call!
                //randomVocEnumerator = ((IEnumerable<VocFileDS.VocsRow>)vocArr).GetEnumerator();
            }

            foreach (VocFileDS.VocsRow row in vocArr)
            {
                vocList.Add(new Voc(row));
            }

            return vocList;
        }

        /// <summary>
        /// Function used to collect information about the results of a test
        /// </summary>
        /// <param name="vocID">The voc ID.</param>
        /// <param name="result">if set to <c>true</c> [result].</param>
        public void NotifyResult(Int32 vocID, Boolean result)
        {
            VocFileDS.VocsRow row = ds.Vocs.FindByID(vocID);
            if (row != null)
            {
                row.TimesRun++;
                row.LastResult = result;
                if (result)
                {
                    row.TimesPassed++;
                }
            }
        }

        #endregion // Public Functions

        #region Public Properties

        /// <summary>
        /// Gets the underlying dataset representation of the vocs.
        /// TODO: Should not exist or be used!
        /// </summary>
        public VocFileDS DataSet
        {
            get
            {
                return ds;
            }
        }

        /// <summary>
        /// Gets a value indicating whether this instance is dirty.
        /// </summary>
        /// <value>
        ///   <c>true</c> if this instance is dirty; otherwise, <c>false</c>.
        /// </value>
        public Boolean IsDirty
        {
            get
            {
                return ds.HasChanges();
            }
        }

        /// <summary>
        /// Gets the name of the file.
        /// </summary>
        /// <value>
        /// The name of the file.
        /// </value>
        public String FileName
        {
            get;
            private set;
        }

        #endregion //Public Properties

        #region Private Helper Functions

        private void DsRow_Changed(object sender, DataRowChangeEventArgs e)
        {
            OnChanged(EventArgs.Empty);
        }

        protected virtual void OnChanged(EventArgs e)
        {
            if (Changed != null)
            {
                Changed(this, e);
            }
        }

        #endregion // Private Helper Functions
    }
}
