﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using System.Drawing.Printing;
using System.Windows.Forms;
using System.IO;

namespace VocaTest
{
    /// <summary>
    /// Helper class used to print Vocs
    /// </summary>
    public class VocsPrintDocument : PrintDocument
    {
        private Font printFont;
        private List<Vocs.Voc> vocList;
        private IEnumerator<Vocs.Voc> vocListEnumerator;

        /// <summary>
        /// Initializes a new instance of the <see cref="VocsPrintDocument"/> class.
        /// </summary>
        public VocsPrintDocument()
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="VocsPrintDocument"/> class.
        /// </summary>
        /// <param name="vocs">The vocs.</param>
        public VocsPrintDocument(Vocs vocs)
            : this()
        {
            vocList = vocs.GetVocs(false, false);
            this.DocumentName = vocs.FileName;
        }

        /// <summary>
        /// Gets or sets the font.
        /// </summary>
        /// <value>
        /// The font.
        /// </value>
        public Font Font
        {
            get
            {
                return printFont;
            }
            set
            {
                printFont = value;
            }
        }

        /// <summary>
        /// Raises the <see cref="E:System.Drawing.Printing.PrintDocument.BeginPrint"/> event. It is called after the <see cref="M:System.Drawing.Printing.PrintDocument.Print"/> method is called and before the first page of the document prints.
        /// </summary>
        /// <param name="e">A <see cref="T:System.Drawing.Printing.PrintEventArgs"/> that contains the event data.</param>
        protected override void OnBeginPrint(PrintEventArgs e)
        {
            base.OnBeginPrint(e);
            vocListEnumerator = vocList.GetEnumerator();
            printFont = new Font("Verdana", 10);
        }

        /// <summary>
        /// Raises the <see cref="E:System.Drawing.Printing.PrintDocument.EndPrint"/> event. It is called when the last page of the document has printed.
        /// </summary>
        /// <param name="e">A <see cref="T:System.Drawing.Printing.PrintEventArgs"/> that contains the event data.</param>
        protected override void OnEndPrint(PrintEventArgs e)
        {
            base.OnEndPrint(e);
            printFont.Dispose();
        }

        /// <summary>
        /// Raises the <see cref="E:System.Drawing.Printing.PrintDocument.PrintPage"/> event. It is called before a page prints.
        /// </summary>
        /// <param name="e">A <see cref="T:System.Drawing.Printing.PrintPageEventArgs"/> that contains the event data.</param>
        protected override void OnPrintPage(PrintPageEventArgs e)
        {
            base.OnPrintPage(e);

            Graphics gdiPage = e.Graphics;
            Int32 leftMargin = e.MarginBounds.Left;
            Int32 topMargin = e.MarginBounds.Top;
            float lineHeight = printFont.GetHeight(gdiPage);
            float linesPerPage = e.MarginBounds.Height / lineHeight;
            Int32 lineWidth = e.MarginBounds.Width;
            int lineCount = 0;

            Int32 x = leftMargin;
            float y = topMargin + (lineCount * lineHeight);

            // Add headers
            Font headerFont = new Font(printFont, FontStyle.Bold);
            gdiPage.DrawString("Language1", headerFont, Brushes.Black, x, y);
            gdiPage.DrawString("Language2", headerFont, Brushes.Black, x + lineWidth / 2, y);
            lineCount++;

            // Print each line of the file.
            while ( (lineCount < linesPerPage) && vocListEnumerator.MoveNext() )
            {
                y = topMargin + (lineCount * lineHeight);
                
                if (lineCount % 2 == 0)
                {
                    gdiPage.DrawLine(new Pen(Brushes.LightSkyBlue, lineHeight), new Point(x, (Int32)(y + lineHeight / 2)), new Point(x + lineWidth, (Int32)(y + lineHeight / 2)));
                }

                gdiPage.DrawString(vocListEnumerator.Current.Language1, printFont, Brushes.Black, x, y);
                gdiPage.DrawString(vocListEnumerator.Current.Language2, printFont, Brushes.Black, x + lineWidth / 2, y);
                
                lineCount++;
            }

            // If more lines exist, print another page.
            if (vocListEnumerator.Current != null)
            {
                e.HasMorePages = true;
            }
            else
            {
                e.HasMorePages = false;
            }
        }
    }
}
